@@ -21,6 +21,7 @@
 # THE SOFTWARE.
 
 import dataiter
+import datetime
 import itertools
 import json
 import numpy as np
@@ -41,7 +42,7 @@ class DataFrameColumn(Vector):
     """
 
     def __new__(cls, object, dtype=None, nrow=None):
-        object = [object] if np.isscalar(object) else object
+        object = cls._sequencify(object)
         column = Vector(object, dtype)
         if nrow is not None and nrow != column.length:
             if column.length != 1 or nrow < 1:
@@ -73,6 +74,19 @@ class DataFrameColumn(Vector):
         """
         return self.length
 
+    @classmethod
+    def _sequencify(cls, object):
+        if isinstance(object, (list, tuple, np.ndarray)):
+            return object
+        if (object is None or
+            np.isscalar(object) or
+            isinstance(object, (bytes, bool, float, int, str, datetime.date, datetime.datetime))):
+            return [object]
+        if hasattr(object, "__iter__"):
+            # Evaluate generator or iterator.
+            return list(object)
+        raise ValueError(f"Unexpected type: {type(object)}")
+
 
 class DataFrame(dict):
 
@@ -189,15 +203,18 @@ class DataFrame(dict):
         """
         Return rows with no matches in `other`.
 
-        `by` are column names, by which to look for matching rows.
+        `by` are column names, by which to look for matching rows, or tuples of
+        column names if the correspoding column name differs between `self` and
+        `other`.
 
         >>> # All listings that don't have reviews
         >>> listings = di.DataFrame.read_csv("data/listings.csv")
         >>> reviews = di.DataFrame.read_csv("data/listings-reviews.csv")
         >>> listings.anti_join(reviews, "id")
         """
-        other = other.unique(*by)
-        found, src = self._get_join_indices(other, *by)
+        by1, by2 = self._split_by(*by)
+        other = other.unique(*by2)
+        found, src = self._get_join_indices(other, by1, by2)
         for colname, column in self.items():
             yield colname, np.delete(column, found)
 
@@ -348,7 +365,9 @@ class DataFrame(dict):
         ones. If there are multiple matches, the first one will be used. For
         rows, for which matches are not found, missing values are added.
 
-        `by` are column names, by which to look for matching rows.
+        `by` are column names, by which to look for matching rows, or tuples of
+        column names if the correspoding column name differs between `self` and
+        `other`.
 
         >>> listings = di.DataFrame.read_csv("data/listings.csv")
         >>> reviews = di.DataFrame.read_csv("data/listings-reviews.csv")
@@ -359,10 +378,10 @@ class DataFrame(dict):
         b = other.filter_out(np.isin(other._id_, a._id_))
         return a.rbind(b).unselect("_id_")
 
-    def _get_join_indices(self, other, *by):
-        other_ids = list(zip(*[other[x] for x in by]))
+    def _get_join_indices(self, other, by1, by2):
+        other_ids = list(zip(*[other[x] for x in by2]))
         other_by_id = {other_ids[i]: i for i in range(other.nrow)}
-        self_ids = zip(*[self[x] for x in by])
+        self_ids = zip(*[self[x] for x in by1])
         src = map(lambda x: other_by_id.get(x, -1), self_ids)
         src = np.fromiter(src, np.int, count=self.nrow)
         found = np.where(src > -1)
@@ -396,17 +415,21 @@ class DataFrame(dict):
         matching ones. If there are multiple matches, the first one will be
         used.
 
-        `by` are column names, by which to look for matching rows.
+        `by` are column names, by which to look for matching rows, or tuples of
+        column names if the correspoding column name differs between `self` and
+        `other`.
 
         >>> listings = di.DataFrame.read_csv("data/listings.csv")
         >>> reviews = di.DataFrame.read_csv("data/listings-reviews.csv")
         >>> listings.inner_join(reviews, "id")
         """
-        other = other.unique(*by)
-        found, src = self._get_join_indices(other, *by)
+        by1, by2 = self._split_by(*by)
+        other = other.unique(*by2)
+        found, src = self._get_join_indices(other, by1, by2)
         for colname, column in self.items():
             yield colname, column[found].copy()
         for colname, column in other.items():
+            if colname in by2: continue
             if colname in self: continue
             yield colname, column[src[found]].copy()
 
@@ -419,17 +442,21 @@ class DataFrame(dict):
         are multiple matches, the first one will be used. For rows, for which
         matches are not found, missing values are added.
 
-        `by` are column names, by which to look for matching rows.
+        `by` are column names, by which to look for matching rows, or tuples of
+        column names if the correspoding column name differs between `self` and
+        `other`.
 
         >>> listings = di.DataFrame.read_csv("data/listings.csv")
         >>> reviews = di.DataFrame.read_csv("data/listings-reviews.csv")
         >>> listings.left_join(reviews, "id")
         """
-        other = other.unique(*by)
-        found, src = self._get_join_indices(other, *by)
+        by1, by2 = self._split_by(*by)
+        other = other.unique(*by2)
+        found, src = self._get_join_indices(other, by1, by2)
         for colname, column in self.items():
             yield colname, column.copy()
         for colname, column in other.items():
+            if colname in by2: continue
             if colname in self: continue
             value = column.missing_value
             dtype = column.missing_dtype
@@ -517,6 +544,20 @@ class DataFrame(dict):
         """
         print(self.to_string(max_rows, max_width))
 
+    def print_missing_counts(self):
+        """
+        Print counts of missing values by column.
+
+        >>> data = di.DataFrame.read_csv("data/listings.csv")
+        >>> data.print_missing_counts()
+        """
+        print("Missing counts:")
+        for name in self.colnames:
+            n = self[name].is_missing().sum()
+            if n == 0: continue
+            pc = 100 * n / self.nrow
+            print(f"... {name}: {n} ({pc:.1f}%)")
+
     @deco.new_from_generator
     def rbind(self, *others):
         """
@@ -621,15 +662,18 @@ class DataFrame(dict):
         """
         Return rows with matches in `other`.
 
-        `by` are column names, by which to look for matching rows.
+        `by` are column names, by which to look for matching rows, or tuples of
+        column names if the correspoding column name differs between `self` and
+        `other`.
 
         >>> # All listings that have reviews
         >>> listings = di.DataFrame.read_csv("data/listings.csv")
         >>> reviews = di.DataFrame.read_csv("data/listings-reviews.csv")
         >>> listings.semi_join(reviews, "id")
         """
-        other = other.unique(*by)
-        found, src = self._get_join_indices(other, *by)
+        by1, by2 = self._split_by(*by)
+        other = other.unique(*by2)
+        found, src = self._get_join_indices(other, by1, by2)
         for colname, column in self.items():
             yield colname, column[found].copy()
 
@@ -679,6 +723,11 @@ class DataFrame(dict):
         for colname, column in self.items():
             yield colname, column[indices].copy()
 
+    def _split_by(self, *by):
+        by1 = [x if isinstance(x, str) else x[0] for x in by]
+        by2 = [x if isinstance(x, str) else x[1] for x in by]
+        return by1, by2
+
     def tail(self, n=None):
         """
         Return the last `n` rows.
@@ -735,7 +784,7 @@ class DataFrame(dict):
         """
         if not self: return ""
         max_rows = dataiter.PRINT_MAX_ROWS if max_rows is None else max_rows
-        max_width = dataiter.PRINT_MAX_WIDTH if max_width is None else max_width
+        max_width = util.get_print_width() if max_width is None else max_width
         n = min(self.nrow, max_rows)
         columns = {colname: util.pad(
             [colname] +
