@@ -47,6 +47,12 @@ class ListOfDicts(list):
     Usually, if you see this warning, you'll want to call :meth:`deepcopy`
     to create a new, completely independent object.
 
+    List of dicts is a subclass of list. This means that if you need fast
+    in-place methods instead of the regular ones that return shallow copies,
+    you can use those from the list baseclass. A common example is appending
+    items one by one in a for loop: instead of ``data = data.append(item)``,
+    you can do ``list.append(data, item)``.
+
     Contained dicts are upon initialization converted to
     ``attd.AttributeDict``, which is a simple subclass of ``dict`` that
     provides attribute access to dict keys. This means that you can access keys
@@ -163,17 +169,20 @@ class ListOfDicts(list):
         """
         Return items with no matches in `other`.
 
-        `by` are keys, by which to look for matching items.
+        `by` are keys, by which to look for matching items, or tuples of keys
+        if the correspoding key differs between `self` and `other`.
 
         >>> # All listings that don't have reviews
         >>> listings = di.ListOfDicts.read_json("data/listings.json")
         >>> reviews = di.ListOfDicts.read_json("data/listings-reviews.json")
         >>> listings.anti_join(reviews, "id")
         """
-        extract = operator.itemgetter(*by)
-        other_ids = set(map(extract, other))
+        by1, by2 = self._split_by(*by)
+        extract1 = operator.itemgetter(*by1)
+        extract2 = operator.itemgetter(*by2)
+        other_ids = set(map(extract2, other))
         for item in self:
-            if extract(item) not in other_ids:
+            if extract1(item) not in other_ids:
                 yield item
 
     @deco.new_from_generator
@@ -293,7 +302,8 @@ class ListOfDicts(list):
         there are multiple matches, the first one will be used. For items, for
         which matches are not found, no keys are added.
 
-        `by` are keys, by which to look for matching items.
+        `by` are keys, by which to look for matching items, or tuples of keys
+        if the correspoding key differs between `self` and `other`.
 
         >>> listings = di.ListOfDicts.read_json("data/listings.json")
         >>> reviews = di.ListOfDicts.read_json("data/listings-reviews.json")
@@ -335,18 +345,23 @@ class ListOfDicts(list):
         `inner_join` keeps only items found in both lists, merging matching
         ones. If there are multiple matches, the first one will be used.
 
-        `by` are keys, by which to look for matching items.
+        `by` are keys, by which to look for matching items, or tuples of keys
+        if the correspoding key differs between `self` and `other`.
 
         >>> listings = di.ListOfDicts.read_json("data/listings.json")
         >>> reviews = di.ListOfDicts.read_json("data/listings-reviews.json")
         >>> listings.inner_join(reviews, "id")
         """
-        extract = operator.itemgetter(*by)
-        other_by_id = {extract(x): x for x in reversed(other)}
+        by1, by2 = self._split_by(*by)
+        extract1 = operator.itemgetter(*by1)
+        extract2 = operator.itemgetter(*by2)
+        other_by_id = {extract2(x): x for x in reversed(other)}
         for item in self:
-            id = extract(item)
+            id = extract1(item)
             if id in other_by_id:
-                item.update(other_by_id[id])
+                new = other_by_id[id]
+                new = {k: v for k, v in new.items() if k not in by2}
+                item.update(new)
                 yield item
 
     @deco.new_from_generator
@@ -375,16 +390,21 @@ class ListOfDicts(list):
         `other`. If there are multiple matches, the first one will be used. For
         items, for which matches are not found, no keys are added.
 
-        `by` are keys, by which to look for matching items.
+        `by` are keys, by which to look for matching items, or tuples of keys
+        if the correspoding key differs between `self` and `other`.
 
         >>> listings = di.ListOfDicts.read_json("data/listings.json")
         >>> reviews = di.ListOfDicts.read_json("data/listings-reviews.json")
         >>> listings.left_join(reviews, "id")
         """
-        extract = operator.itemgetter(*by)
-        other_by_id = {extract(x): x for x in reversed(other)}
+        by1, by2 = self._split_by(*by)
+        extract1 = operator.itemgetter(*by1)
+        extract2 = operator.itemgetter(*by2)
+        other_by_id = {extract2(x): x for x in reversed(other)}
         for item in self:
-            item.update(other_by_id.get(extract(item), {}))
+            new = other_by_id.get(extract1(item), {})
+            new = {k: v for k, v in new.items() if k not in by2}
+            item.update(new)
             yield item
 
     def _mark_obsolete(self):
@@ -461,6 +481,23 @@ class ListOfDicts(list):
         """
         print(self.to_string(max_items))
 
+    def print_missing_counts(self):
+        """
+        Print counts of missing values by key.
+
+        Both keys entirely missing and keys with a value of ``None`` are
+        considered missing.
+
+        >>> data = di.ListOfDicts.read_csv("data/listings.json")
+        >>> data.print_missing_counts()
+        """
+        print("Missing counts:")
+        for key in util.unique_keys(list(itertools.chain(*self))):
+            n = sum(x.get(key, None) is None for x in self)
+            if n == 0: continue
+            pc = 100 * n / len(self)
+            print(f"... {key}: {n} ({pc:.1f}%)")
+
     @classmethod
     def read_csv(cls, fname, encoding="utf_8", header=True, sep=","):
         """
@@ -542,17 +579,20 @@ class ListOfDicts(list):
         """
         Return items with matches in `other`.
 
-        `by` are keys, by which to look for matching items.
+        `by` are keys, by which to look for matching items, or tuples of keys
+        if the correspoding key differs between `self` and `other`.
 
         >>> # All listings that have reviews
         >>> listings = di.ListOfDicts.read_json("data/listings.json")
         >>> reviews = di.ListOfDicts.read_json("data/listings-reviews.json")
         >>> listings.semi_join(reviews, "id")
         """
-        extract = operator.itemgetter(*by)
-        other_ids = set(map(extract, other))
+        by1, by2 = self._split_by(*by)
+        extract1 = operator.itemgetter(*by1)
+        extract2 = operator.itemgetter(*by2)
+        other_ids = set(map(extract2, other))
         for item in self:
-            if extract(item) in other_ids:
+            if extract1(item) in other_ids:
                 yield item
 
     def sort(self, **key_dir_pairs):
@@ -577,6 +617,11 @@ class ListOfDicts(list):
             data = sorted(data, key=sort_key, reverse=dir < 0)
         return self._new(data)
 
+    def _split_by(self, *by):
+        by1 = [x if isinstance(x, str) else x[0] for x in by]
+        by2 = [x if isinstance(x, str) else x[1] for x in by]
+        return by1, by2
+
     def tail(self, n=None):
         """
         Return the last `n` items.
